// ==UserScript==
// @include       http://one.ubuntu.com/music/player/*
// @include       http://edge.one.ubuntu.com/music/player/*
// @require       utils.js
// ==/UserScript==

window.Unity = external.getUnityObject(1);

function isCorrectPage() {
	var player = document.getElementById("media-player");
	return player ? true : false;
}

function getNowPlayingInfo() {
	function parseInfoSafe(xpathQuery, selector, context) {
		var val = _("Unknown");

		try {
			val = document.evaluate(xpathQuery, context, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue.innerHTML;
		} catch (err) {
			var valElement = context.querySelector(selector);
			if (valElement !== null) {
				val = valElement.innerHTML;
			}
		}
		return val;
	}

	var nowPlayingDiv = document.evaluate('//div[@class="now-playing"]/div[@class="current-song"]', document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;

	if (nowPlayingDiv === null) {
		return null;
	}

	var title = parseInfoSafe('p', '.title', nowPlayingDiv);
	var artist = parseInfoSafe('dl/dd[@class="artist"]', '.artist', nowPlayingDiv);
	var album = parseInfoSafe('dl/dd[@class="album"]', '.album', nowPlayingDiv);

	// the album art bit is a little bit different so our safe parse method doesn't work.
	var artLocation = "";
	try {
		artLocation = document.evaluate('div[@class="albumart"]/img', nowPlayingDiv, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue.src;
	} catch (err) {
		var i, artElement = nowPlayingDiv.querySelector('.albumart');
		for (i = 0; i < artElement.childNodes.length; i++) {
			var child = artElement.childNodes[i];
			if (child.nodeName === "IMG") {
				artLocation = child.src;
			}
		}
	}

	return {
		title: title,
		album: album,
		artist: artist,
		artLocation: artLocation
	};
}

function setNextPreviousState() {
	/*
	these bits aren't working anyway so they're just commented out.
	var nextButton = document.getElementById('next');
	Unity.MediaPlayer.setCanGoNext(nextButton.className.search("disabled") == -1 ? 0 : 1);

	var prevButton = document.getElementById('prev');
	Unity.MediaPlayer.setCanGoPrevious(prevButton.className.search("disabled") == -1 ? 0 : 1);
	*/
}

function isPlaying() {
	var button = document.getElementById('play');
	return button.style.display === "none";
}

function setTrack() {
	var trackInfo = getNowPlayingInfo();

	setTimeout(setTrack, 1500);

	if (trackInfo) {
		Unity.MediaPlayer.setTrack(trackInfo);
		Unity.MediaPlayer.setPlaybackState(isPlaying() ? Unity.MediaPlayer.PlaybackState.PLAYING : Unity.MediaPlayer.PlaybackState.PAUSED);
	}
}

function musicPlayerSetup() {
	Unity.MediaPlayer.onPlayPause(wrapCallback(function () {
		var button = document.getElementById('play');
		var playing = false;

		if (isPlaying()) {
			button = document.getElementById('stop');
			playing = true;
		}

		launchClickEvent(button);
		Unity.MediaPlayer.setPlaybackState(playing ? Unity.MediaPlayer.PlaybackState.PLAYING : Unity.MediaPlayer.PlaybackState.PAUSED);
	}));

	Unity.MediaPlayer.onNext(wrapCallback(function () {
		var nextButton = document.getElementById('next');
		launchClickEvent(nextButton);
		setNextPreviousState();
	}));

	Unity.MediaPlayer.onPrevious(wrapCallback(function () {
		var previousButton = document.getElementById('prev');
		launchClickEvent(previousButton);
		setNextPreviousState();
	}));

	setTrack();
}

if (isCorrectPage()) {
	Unity.init({ name: "UbuntuOne",
		     domain: 'one.ubuntu.com',
		     homepage: 'https://one.ubuntu.com',
		     iconUrl: "icon://ubuntuone",
		     onInit: wrapCallback(musicPlayerSetup) });
}
